@@ -31,7 +31,11 @@
 
 #define SILENT_WINDOW_OPEN_LIMIT 5
 
+#include <gnome.h>
+
+#include <string.h>
 #include <signal.h>
+#include <sys/wait.h>
 
 #include "gsearchtool.h"
 #include "gsearchtool-support.h"
@@ -122,8 +126,8 @@ click_find_cb (GtkWidget	*widget,
 		gnome_appbar_pop (GNOME_APPBAR (interface.status_bar));
 		gnome_appbar_push (GNOME_APPBAR (interface.status_bar), 
 				   _("Searching..."));
-		search_command.timeout = gtk_timeout_add (100, update_progress_bar, 
-							  NULL); 
+		search_command.timeout = g_timeout_add (100, update_progress_bar, 
+						  	NULL); 
 		search_command.show_hidden_files = FALSE;
 		
 		spawn_search_command (command);
@@ -151,11 +155,19 @@ click_stop_cb (GtkWidget 	*widget,
 	gtk_widget_set_sensitive (interface.find_button, TRUE);
 }
 
+void
+size_allocate_cb (GtkWidget	*widget,
+		  GtkAllocation *allocation,
+	      	  gpointer 	data)
+{
+	gtk_widget_set_size_request (interface.add_button,
+			      	     allocation->width,
+			             allocation->height);
+}
 void
 add_constraint_cb (GtkWidget 	*widget, 
 		   gpointer 	data)
 {
-	SearchConstraint *constraint = g_new(SearchConstraint,1);
 	GtkWidget *menu;
 	GtkWidget *item;
 	
@@ -163,7 +175,7 @@ add_constraint_cb (GtkWidget 	*widget,
 	item = gtk_menu_get_active (GTK_MENU(menu));
 	
 	if (GTK_WIDGET_SENSITIVE(item) == TRUE) {
-		add_constraint (interface.selected_constraint, NULL);
+		add_constraint (interface.selected_constraint, NULL, FALSE);
 	}
 }
 
@@ -185,6 +197,7 @@ remove_constraint_cb (GtkWidget 	*widget,
 		g_list_remove(interface.selected_constraints, constraint);
 	
 	set_constraint_selected_state (constraint->constraint_id, FALSE);	
+	set_constraint_gconf_boolean (constraint->constraint_id, FALSE);	
 }
 
 void
@@ -220,23 +233,11 @@ void
 constraint_entry_changed_cb (GtkWidget 	*widget, 
 			     gpointer 	data)
 {
-	static gchar *file_is_named_string;
 	static gchar *look_in_folder_string;
 	
 	if (GTK_WIDGET_VISIBLE (interface.main_window) == FALSE) {
 		return;
 	}
-	
-	if (GTK_WIDGET_VISIBLE (interface.additional_constraints) == FALSE) {
-	
-		file_is_named_string = 
-			(gchar *) gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(interface.file_is_named_entry))));
-	
-		if (strlen (file_is_named_string) == 0) {
-			gtk_widget_set_sensitive (interface.find_button, FALSE);
-			return;
-		}
-	} 
 		
 	look_in_folder_string = 
 		gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(interface.look_in_folder_entry), FALSE);
@@ -474,6 +475,18 @@ file_button_press_event_cb (GtkWidget 		*widget,
 	return !(row_selected_by_button_press_event);
 }
 
+gboolean
+file_key_press_event_cb  (GtkWidget 		*widget, 
+		    	  GdkEventKey	 	*event, 
+		     	  gpointer 		data)
+{		
+	if (event->keyval == GDK_space || event->keyval == GDK_Return) {
+		open_file_cb (widget, data);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 gboolean
 file_button_release_event_cb (GtkWidget 	*widget, 
 		      	      GdkEventButton 	*event, 
@@ -753,26 +766,50 @@ save_results_cb (GtkFileSelection *selector,
 	
 	interface.save_results_file = (gchar *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(interface.file_selector));
 	
-	if (access (interface.save_results_file, F_OK) == 0) {
-		GtkWidget *dialog;
-		gint response;
+	if (g_file_test (interface.save_results_file, G_FILE_TEST_EXISTS) == TRUE) {
 		
-		dialog = gtk_message_dialog_new
-			(GTK_WINDOW (interface.file_selector),
-			 0 /* flags */,
-			 GTK_MESSAGE_QUESTION,
-			 GTK_BUTTONS_YES_NO,
-			 _("File \"%s\" already exists. Overwrite?"),
-			 interface.save_results_file);
-		gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (interface.main_window));
-		response = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (g_file_test (interface.save_results_file, G_FILE_TEST_IS_DIR) == TRUE) {
+			GtkWidget *dialog;
+			gchar *error_msg = g_strdup_printf (_("Cannot save search results.\n\"%s\" is a folder."),
+							      interface.save_results_file);
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (interface.main_window),
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_OK,
+					error_msg);
+
+			g_signal_connect (G_OBJECT (dialog),
+					"response",
+					G_CALLBACK (gtk_widget_destroy), NULL);
+					
+			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+			gtk_widget_show (dialog);
+			g_free (error_msg);
+			
+			return;	
+		}
+		else {
+			GtkWidget *dialog;
+			gint response;
 		
-		gtk_widget_destroy (GTK_WIDGET(dialog));
+			dialog = gtk_message_dialog_new
+				(GTK_WINDOW (interface.file_selector),
+				 0 /* flags */,
+				 GTK_MESSAGE_QUESTION,
+				 GTK_BUTTONS_YES_NO,
+				 _("File \"%s\" already exists. Overwrite?"),
+				 interface.save_results_file);
+			gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW (interface.main_window));
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
 		
-		if (response != GTK_RESPONSE_YES) return;
+			gtk_widget_destroy (GTK_WIDGET(dialog));
+		
+			if (response != GTK_RESPONSE_YES) return;
+		}
 	}
 	
-	if (fp = fopen(interface.save_results_file, "w")) {
+	if ((fp = fopen (interface.save_results_file, "w")) != NULL) {
 	
 		if (gtk_tree_model_get_iter_root(GTK_TREE_MODEL(store), &iter)) {
 			
@@ -826,11 +863,12 @@ save_session_cb (GnomeClient 	    *client,
 		 gint 		    fast,
  	      	 gpointer 	    client_data)
 {
- 	gchar *argv[] = { NULL };
- 
- 	argv[0] = (gchar *) client_data;
- 	gnome_client_set_clone_command (client, 1, argv);
- 	gnome_client_set_restart_command (client, 1, argv);	
+	char **argv;
+	int argc;
+
+	set_clone_command (&argc, &argv, client_data);
+	gnome_client_set_clone_command (client, argc, argv);
+	gnome_client_set_restart_command (client, argc, argv);	
 }
 
 gboolean
@@ -838,13 +876,13 @@ key_press_cb (GtkWidget    	*widget,
 	      GdkEventKey	*event, 
 	      gpointer 		data)
 {
-	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
 
 	if (event->keyval == GDK_Escape) {
 		if (search_command.running == RUNNING) {
 			click_stop_cb (widget, NULL);
 		}
-		else {
+		else if (search_command.not_running_timeout == FALSE) {
 			quit_cb (widget, NULL);
 		}
 	}
@@ -885,7 +923,7 @@ look_in_folder_key_press_cb (GtkWidget    	*widget,
 		     	     GdkEventKey	*event, 
 		     	     gpointer 		data)
 {
-	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
 
 	if (event->keyval == GDK_Return)
 	{
@@ -896,3 +934,10 @@ look_in_folder_key_press_cb (GtkWidget    	*widget,
 	}
 	return FALSE;
 }
+
+gboolean
+not_running_timeout_cb (gpointer data)
+{
+	search_command.not_running_timeout = FALSE;
+	return FALSE;
+}
